// import dotenv from "dotenv";

import axios, { AxiosError, AxiosInstance } from "axios";
import { Event, EventBuilder, Response, Ticket } from "./model/model";
import { httpClient } from "./utils/httpClient";

// dotenv.config();

class MentaSDK {
  private httpClient: httpClient;

  constructor(private apiKey: string, private environment: 'production' | 'sandbox' | 'local') {
    this.httpClient = new httpClient(apiKey, MentaSDK.getBaseUrl(environment))
  }

  private static getBaseUrl(environment: 'production' | 'sandbox' | 'local') {
    switch (environment) {
      case 'production':
        return 'https://api.mentatickets.com';
      case 'sandbox':
        return 'https://api.sandbox.mentatickets.com';
      case 'local':
        return 'http://localhost:4405' // TODO: change this to api.mentatickets.local
      default:
        return 'https://api.sandbox.mentatickets.com';
    }
  }

  /**
   * @param event The event object to be created. We recommend to create it using the EventBuilder.
   * @returns A promise with the response from the API. Smart contract deployment will occur in the background.
   * @see https://docs.mentatickets.com/docs/creating-events
   */
  public async createEvent(event: Event): Promise<Response> {
    const call = await this.httpClient.post('/events', event)
    return call
  }

  /**
   * @param eventId The ID of the event you are using in your system.
   * @param showId (optional) The ID of the event (if many) you are using in your system.
   * @returns Public URL to Menta's Marketplace
   * @see https://docs.mentatickets.com/docs/resale-flow
   */
  public async generateResaleUrl(eventId: string, showId?: string): Promise<Response> {
    const params = {
      externalReferenceEventId: eventId,
      showId: showId
    } as Record<any, any>

    return await this.httpClient.get(`/marketplace/buyingflow/url/entrypoint?${(new URLSearchParams(params)).toString()}`)
  }

  /**
   * @param eventId The ID of the event you are using in your system.
   * @param ticketId The unique ID of the ticket to be admitted.
   * @returns Confirmation of the ticket admission.
   * @see https://docs.mentatickets.com/docs/admitting-tickets
   */
  public async ticketAdmitted(eventId: string, ticketId: string): Promise<Response> {
    const url = `/events/${eventId}/tickets/${ticketId}/eventAdmision`
    return await this.httpClient.get(url)
  }

  /**
   * Cancels a show. ShowId must be provided if event has more than more shows.
   * @param eventId The ID of the event you are using in your system.
   * @param showId (Optional) The ID of the show, if the event has more than one.
   * @returns Confirmation of the show cancellation.
   */
  public async cancelShow(eventId: string, showId: string): Promise<Response> {
    let url = `/events/${eventId}/cancellation`
    if (showId) {
      url = url + `?showId=${showId}`
    }
    return await this.httpClient.post(url, {})
  }

  public async createTickets(tickets: Ticket[]): Promise<Response> {
    // lógica para crear un ticket con el email y el objeto especificados
    const url = `/events/tickets/tokenize`;
    return await this.httpClient.post(url, tickets)
  }

  public shouldShowQRCode(email: string, ticketId: string): boolean {
    // lógica para determinar si se debe mostrar el código QR con el email y el ticketId especificados
    return false
  }
}

export const initializeMenta = (apiKey: string, environment: 'production' | 'sandbox' | 'local' = 'production') => {
  return new MentaSDK(apiKey, environment);
}