
export type Response = {
    status: number,
    data: string
};

export type TicketOption = {
    ticketId: string,
    title: string,
    price: number,
    currency: string,
    description: string,
}

export type Show = {
    showId: string,
    title: string,
    status: string,
    date: Date,
    ticketOptions: TicketOption[],
    images: { url: string, kind: 'main' | 'secondary' }[]
}

export type Location = {
    city: string,
    state: string,
    venueName: string,
    address: string,
    country: string,
}

export type Event = {
    externalReferenceId: string,
    title: string,
    shows: Show[],
    location: Location,
    description: string,
}

export type Ticket = {
    ticketOptionId: string,
    showId: string,
    externalReferenceEventId: string,
    ticketId: string,
    buyer: string,
    ticketAccess: {
        locator: string,
        type: 'URL' | 'API_DYNAMIC_QR' | 'BASE64' | 'OTHER',
    }
}

export class ShowBuilder {
    private show: Show = {} as Show

    public withShowId(showId: string): ShowBuilder {
        this.show.showId = showId;
        return this;
    }

    public withTitle(title: string): ShowBuilder {
        this.show.title = title;
        return this;
    }

    public withDateStart(date: Date): ShowBuilder {
        this.show.date = date;
        return this;
    }

    public addTicketOption(ticketOption: TicketOption): ShowBuilder {
        if (!this.show.ticketOptions) this.show.ticketOptions = [];
        this.show.ticketOptions!.push(ticketOption);
        return this;
    }

    public addImage(url: string, kind: 'main' | 'secondary'): ShowBuilder {
        if (!this.show.images) this.show.images = [];
        this.show.images!.push({ url, kind });
        return this;
    }

    public build(): Show {
        this.show.status = 'LIVE'
        return this.show;
    }
}

export class EventBuilder {
    private event: Event = {} as Event

    public withEventId(eventId: string): EventBuilder {
        this.event.externalReferenceId = eventId;
        return this;
    }

    public withTitle(name: string): EventBuilder {
        this.event.title = name;
        return this;
    }

    public withDescription(description: string): EventBuilder {
        this.event.description = description;
        return this;
    }

    public addShow(show: Show): EventBuilder {
        if (!this.event.shows) this.event.shows = [];
        this.event.shows!.push(show);
        return this;
    }

    public withLocation(city: string, state: string, venueName: string, address: string, country: string): EventBuilder {
        this.event.location = {
            city,
            state,
            venueName,
            address,
            country
        }
        return this;
    }

    public build(): Event {
        return this.event;
    }
}

export class TicketBuilder {
    private ticket: Ticket = {} as Ticket

    /**
     * 
     * @param ticketId The unique identifier of the ticket. Must be unique within the event.
     * @returns TicketBuilder
     * @see https://docs.mentatickets.com/docs/tokenization#ticketId
     */
    public withTicketId(ticketId: string): TicketBuilder {
        this.ticket.ticketId = ticketId;
        return this;
    }

    /**
     * 
     * @param ticketOptionId The unique identifier of the ticket option. Must be unique within the show.
     * @returns 
     * @see https://docs.mentatickets.com/docs/tokenization#ticketOptionId
     */
    public withTicketOptionId(ticketOptionId: string): TicketBuilder {
        this.ticket.ticketOptionId = ticketOptionId;
        return this;
    }

    /**
     * 
     * @param showId The idenfitier of the show.
     * @returns TicketBuilder
     * @see https://docs.mentatickets.com/docs/tokenization#showId
     */
    public withShowId(showId: string): TicketBuilder {
        this.ticket.showId = showId;
        return this;
    }

    /**
     * 
     * @param externalReferenceEventId The ID of the event you use to identify the event in your system.
     * @returns TicketBuilder
     * @see https://docs.mentatickets.com/docs/tokenization#externalReferenceEventId
     */
    public withExternalReferenceEventId(externalReferenceEventId: string): TicketBuilder {
        this.ticket.externalReferenceEventId = externalReferenceEventId;
        return this;
    }

    /**
     * 
     * @param buyer The identifier of the buyer. Currently is only supported email.
     * @returns TicketBuilder
     * @see https://docs.mentatickets.com/docs/tokenization#buyer
     */
    public withBuyer(buyer: string): TicketBuilder {
        this.ticket.buyer = buyer;
        return this;
    }

    /**
     * 
     * @param locator The locator of the ticket. It can be a URL, a QR code, a base64 image, etc. Type param must match with the information provided in this param.
     * @param type The type of the locator. It can be URL, API_DYNAMIC_QR, BASE64 or OTHER.
     * @returns TicketBuilder
     * @see https://docs.mentatickets.com/docs/tokenization#ticket-access
     */
    public withTicketAccess(locator: string, type: 'URL' | 'API_DYNAMIC_QR' | 'BASE64' | 'OTHER'): TicketBuilder {
        this.ticket.ticketAccess = {
            locator,
            type
        }
        return this;
    }

    public build(): Ticket {
        return this.ticket;
    }
}